package ucp;

use IO::Socket;
$RESP_HEADER_LEN = 17;
$OP_HEADER_LEN = 17;

sub new {
	my $class = shift;
	my $self = {};
	($self->{SeqNum}) = 0;
	$self->{Packet};
	$self->{smsh};
	bless $self, $class;
}

sub new_trn_num {
	my $self = shift;
	$self->{SeqNum}++;
	$self->{SeqNum} %= 100;
	return ($self->{Seq_num});
}
sub create_header {
	my $self = shift;
	my ($trn, $len, $o_r, $ot) = @_;
	my $pkg;
	if (length($trn) != 0) {
		$pkg = sprintf("%02d", scalar($trn) );
	}
	else {
		$pkg = sprintf("%02d", ($self->new_trn_num + 0));
	}
	$pkg .= "/";
	$pkg .= sprintf("%05d", $len);
	$pkg .= "/";
	$pkg .= sprintf("%s", $o_r);
	$pkg .= "/";
	$pkg .= sprintf("%02d", $ot);
	return ($pkg);
}

sub create_ack {
	my $self = shift;
	my ($type, $oper, $TRN, $Magic, $SM) = @_;
	if ($type == "+") {
		if ($oper == "01") {
			my $pkt = "A/";
			$tmp = $self->create_header($TRN, length($pkt) + $RESP_HEADER_LEN , "R", $oper) . "/" . $pkt . "/";
			$chk = sprintf("%02X", $self->create_chksum($tmp));
			$vegso = sprintf("%s%s%s%s\n", chr(0x02), $tmp, $chk, chr(0x03));
		}
		else {
			my $pkt = "A//";
			$pkt .= sprintf("%s/", $Magic);
			$tmp = $self->create_header($TRN, length($pkt) + $RESP_HEADER_LEN, "R", $oper) . "/" . $pkt . "/";
			$chk = sprintf("%02X", $self->create_chksum($tmp));
			$vegso = sprintf("%s%s%s%s\n", chr(0x02), $tmp, $chk, chr(0x03));
		}
	}
	else {
		if ($oper == "01") {
			my $pkt = "N/";
			$tmp = $self->create_header($TRN, length($pkt) + $RESP_HEADER_LEN , "R", $oper) . "/" . $pkt . "/";
			$chk = sprintf("%02X", $self->create_chksum($tmp));
			$vegso = sprintf("%s%s%s%s\n", chr(0x02), $tmp, $chk, chr(0x03));
		}
		else {
			my $pkt = "N/";
			$pkt .= sprintf("%s//", $Magic);
			$tmp = $self->create_header($TRN, length($pkt) + $RESP_HEADER_LEN, "R", $oper) . "/" . $pkt . "/";
			$chk = sprintf("%02X", $self->create_chksum($tmp));
			$vegso = sprintf("%s%s%s%s\n", chr(0x02), $tmp, $chk, chr(0x03));
		}
	}
	return ($vegso);	
}

sub send_ack {
	my $self = shift;
	my ($type) = @_;
	if ($type == "+") {
		$self->write_pkt($self->create_ack("+", $self->get_oper, $self->get_TRN, "", ""));
		return 0;
	}
	elsif ($type == "-") {
		$self->write_pkt($self->create_ack("-", $self->get_oper, $self->get_TRN, "", ""));
		return 0;
	}
	return 1;
}


sub op52 {
	my $self = shift;
	parse_pkt($self->{Packet});
	return 1;
}
	
sub create_chksum {
	my $self = shift;
	my ($s) = @_;
	(@a) = split(//, $s);
	$t = 0;
	foreach $a (@a) {
		$t += ord($a);
	}
	$t2 = $t & 255;
	return ($t2);
}
	
sub op51 {
	my $self = shift;
	my ($prior, $sched, $valid, $regist, $src, $dest, $regist, $MSG) = @_;
	my $pkg;
	$pkg = sprintf("%s", $dest);
	$pkg .= "/";
	$pkg .= sprintf("%s", $src);
	$pkg .= "/";
	$pkg .= "/";	#AC
	if ($regist >= 1) {
		$pkg .= "1/";
		$pkg .= sprintf("%s", $src);
		$pkg .= "/"; #NAdC
		$pkg .= sprintf("%1d", $regist);
		$pkg .= "/";
		$pkg .= sprintf("0100");
#		$pkg .= sprintf("0539");
		$pkg .= "/";
	}
	else {
		$pkg .= "////";
	}
	$pkg .= "///"; 	#LRq, LRAd, LPID
	if ($sched != "") {
		$pkg .= "1/";	#DD
		$pkg .= $sched;	#DDT
		$pkg .= "/";
	}
	else {
		$pkg .= "/";	#DD
		$pkg .= "/";	#DDT
	}
	$pkg .= (($valid != "") ? $valid : "");	#VP
	$pkg .= "/";	
	$pkg .= "/";	#RPID
	$pkg .= "/";	#SCTS not used
	$pkg .= "/";	#Dst not used
	$pkg .= "/";	#Rsn not used
	$pkg .= "/";	#DSCTS not used
	$pkg .= "3/";	#MT message type 3
	$pkg .= "/";	#NB
 	@a = split(//, $MSG);
	foreach $a (@a) {
		$pkg .= sprintf("%02X", ord($a));
	}
	$pkg .= "/";
	$pkg .= "/";	#MMS
	$pkg .= (($prior == 1) ? "1/" : "/");	#PR
	$pkg .= "/";	#DCs
	$pkg .= "/";	#MCLs
	$pkg .= "/";	#RPI
	$pkg .= "/";	#CPg
	$pkg .= "/";	#RPLy
	$pkg .= "/";	#OTOA
	$pkg .= "/";	#HPLMN
	$pkg .= "/";	#XSer
	$pkg .= "/";	#RES4
	return $pkg;
}


#opid = "MS" for mobile station
#opid = "PC" for PC application
sub op60 {
	my $self = shift;
	my ($saddr, $ston, $snpi, $optype, $pwd, $npwd, $opid) = @_;
	my $pkt;
	$pkt = sprintf("%s/", $saddr);
	$pkt .= (($ston == "") ? "/" : sprintf("%d/", $ston));
	$pkt .= (($snpi == "") ? "/" : sprintf("%d/", $snpi));
	$pkt .= sprintf("%d/", $optype);	
 	@a = split(//, $pwd);
	foreach $a (@a) {
		$pkt .= sprintf("%02X", ord($a));
	}
	$pkt .= "/"; 	#pwd
	if ($npwd != "") {
		@a = split(//, $npwd);
		foreach $a (@a) {
			$pkt .= sprintf("%02X", ord($a));
		}
	}
	$pkt .= "/"; 	#npwd
	$pkt .= "0100/";#vers
	$pkt .= "/";	#ladc
	$pkt .= "/";	#lton
	$pkt .= "/";	#lnpi
#	$pkt .= "39/";	#opid
#	if (($opid) =~ /PC/) {
#		$pkt .= "39/";
#	}
#	else {
#		$pkt .= "00/";
#	}
	$pkt .= (($opid =~ /PC/) ? sprintf("39/")  : sprintf("00/"));
#	ez mer nem megy?!?!?
	$pkt .= "/";	#res
	return ($pkt);
}

sub open_session {
	my $self = shift;
	my ($saddr, $ston, $snpi, $pwd, $opid) = @_;
	$data = $self->op60($saddr, $ston, $snpi, "1", $pwd, "", $opid);
	$tmp = $self->create_header("", length($data) + $OP_HEADER_LEN  -1 , "O", 60) . "/" . $data ; 
	$chk = sprintf("%02X", $self->create_chksum($tmp));
	$vegso = sprintf("%s%s%s%s\n", chr(0x02), $tmp, $chk, chr(0x03));
	$self->write_pkt($vegso);
	if ($self->read_pkt != 1) {
		if ($self->{Packet}->{O_R} == "R") {
#			$self->print_resp;
			return (($self->{Packet}->{ACK} =~ /A/) ? 0: 1);
		}
		return 2;
	}
	else {
		return 1;
	}
}

sub change_password {
	my $self = shift;
	my ($saddr, $ston, $snpi, $pwd, $npwd, $opid) = @_;
	$data = $self->op60($saddr, $ston, $snpi, "3", $pwd, $npwd, $opid);
	$tmp = $self->create_header("", length($data) + $OP_HEADER_LEN, "O", 60) . "/" . $data . "/";
	$chk = sprintf("%02X", $self->create_chksum($tmp));
	$vegso = sprintf("%s%s%s%s\n", chr(0x02), $tmp, $chk, chr(0x03));
	$self->write_pkt($vegso);
	if (defined($p = $self->read_pkt)) {
		if ($self->{Packet}->{O_R} == "R") {
			$self->parse_resp($p);
			return (($self->{Packet}->{ACK} == "A") ? 0: 1);
		}
		return 2;
	}
	else {
		return 1;
	}
}

sub submit {
	$self = shift;
	my ($prior, $sched, $valid, $regist, $src, $dest, $notif, $MSG) = @_;
	$data = $self->op51($prior, $sched, $valid, $regist, $src, $dest, $notif, $MSG);
	$tmp = $self->create_header("", length($data) + $OP_HEADER_LEN, "O", 51) . "/" . $data . "/";
	$chk = sprintf("%02X", $self->create_chksum($tmp));
	$vegso = sprintf("%s%s%s%s\n", chr(0x02), $tmp, $chk, chr(0x03));
	$self->write_pkt($vegso);
	if (defined($p = $self->read_pkt)) {
		if ($self->{Packet}->{O_R} == "R") {
#			$self->parse_resp($p);
#			$self->print_resp;
			return ((($self->{Packet}->{ACK}) == "A") ? 0 : 1);
		}
		return 2;
	}
	else {
		return 3;
	}
}

sub receive {
	my $self = shift;
	my $pkt;
	if (defined($pkt = $self->read_pkt)) {
		return (selector);
	}
	return 1;
}

sub connect {
	my $self = shift;
	my ($hostname, $port) = @_;
	$self->{smsh} = IO::Socket::INET->new(PeerAddr	=> $hostname,
											PeerPort=> $port,
											Proto	=> 'tcp');
	return ((defined($self->{smsh})) ? 0 : 1);
}

sub read_pkt {
	my $self = shift;
	my $pkt;
	$self->{smsh}->recv($c, 4);			
	if (length($c) != 4) {
		return 1;
	}
	$c =~ s/^.//;
	$pkt = $c;
	($self->{Packet}->{TRN}) =  ($c =~ /^(.{2}).*/);
	$self->{smsh}->recv($c, 5);
	if (length($c) != 5) {
		return 1;
	}
	$pkt .= $c;
	$self->{Packet}->{LEN} = $c; 
	$self->{smsh}->recv($d, $c -7);
	if (length($d) != $c - 7 ) {
		return 1;
	}
	$d =~ s/.$//;
	$pkt .= $d;
	if ($d =~ /^\/O/) {
		$self->{Packet}->{O_R} = "O";

	}
	else {
		$self->{Packet}->{O_R} = "R";
	}
	$d =~ /^\/.\/(.{2})/;
	$self->{Packet}->{OT}  = $1;
	$self->selector($pkt);
	return 0;	
}

sub parse_5xpkt {
	my $self = shift;
	my ($pkt) = @_;
	$self->{Packet} = undef;
	($self->{Packet}->{TRN}, 
	$self->{Packet}->{LEN}, 
	$self->{Packet}->{O_R}, 
	$self->{Packet}->{OT}, 
	$self->{Packet}->{AdC}, 
	$self->{Packet}->{OAdC}, 
	$self->{Packet}->{AC}, 
	$self->{Packet}->{NRq}, 
	$self->{Packet}->{NAdC}, 
	$self->{Packet}->{NT}, 
	$self->{Packet}->{NPID}, 
	$self->{Packet}->{LRq}, 
	$self->{Packet}->{LRAd}, 
	$self->{Packet}->{LPID}, 
	$self->{Packet}->{DD}, 
	$self->{Packet}->{DDT}, 
	$self->{Packet}->{VP}, 
	$self->{Packet}->{RPID}, 
	$self->{Packet}->{SCTS}, 
	$self->{Packet}->{Dst}, 
	$self->{Packet}->{Rsn}, 
	$self->{Packet}->{DSCTS}, 
	$self->{Packet}->{MT}, 
	$self->{Packet}->{NB}, 
	$self->{Packet}->{Msg}, 
	$self->{Packet}->{MMS}, 
	$self->{Packet}->{PR}, 
	$self->{Packet}->{DCs}, 
	$self->{Packet}->{MCLs}, 
	$self->{Packet}->{RPI}, 
	$self->{Packet}->{CPg}, 
	$self->{Packet}->{RPLy}, 
	$self->{Packet}->{OTOA}, 
	$self->{Packet}->{HPLMN}, 
	$self->{Packet}->{XSer}, 
	$self->{Packet}->{RES4}) = split(/\//, $pkt);
}

sub parse_0xpkt {
	my $self = shift;
	my ($pkt) = @_;
	$self->{Packet} = undef;
	($self->{Packet}->{TRN}, 
	$self->{Packet}->{LEN}, 
	$self->{Packet}->{O_R}, 
	$self->{Packet}->{OT}, 
	$self->{Packet}->{AdC}, 
	$self->{Packet}->{OAdC}, 
	$self->{Packet}->{AC}, 
	$self->{Packet}->{MT}, 
	$self->{Packet}->{Msg}) = split(/\//, $pkt);
	$self->{Packet}->{Msg} = $self->un_IA5($self->{Packet}->{Msg});
}

sub un_IA5 {
	my $self = shift;
	my ($msg) = @_;
	my $tmp = "";
	my $out = "";
	while (length($msg) != "") {
		($tmp, $msg) = ($msg =~ /(..)(.*)/);
		$out .= sprintf("%s", chr(hex($tmp)));
	}
	return ($out);
}
				  

sub parse_resp {
	my $self = shift;
	my ($pkt) = @_;
	$self->{Packet} = undef;
	($self->{Packet}->{TRN}, 
	$self->{Packet}->{LEN}, 
	$self->{Packet}->{O_R}, 
	$self->{Packet}->{OT}, 
	$self->{Packet}->{ACK}, 
	$self->{Packet}->{Magic}, 
	$self->{Packet}->{SM}) = split(/\//, $pkt); 
}


sub write_pkt {
	my $self = shift;
	my ($pkt) = @_;
	return ($self->{smsh}->send($pkt) == length($pkt));
}

sub get_originator {
	my $self = shift;
	return ($self->{Packet}->{OAdC});
}

sub get_addressee {
	my $self = shift;
	return ($self->{Packet}->{AdC});
}

sub get_TRN {
	my $self = shift;
	return ($self->{Packet}->{TRN});
}

sub get_sm {
	my $self = shift;
	return ($self->{Packet}->{Msg});
}

sub selector {
	my $self = shift;
	my ($pkt) = @_;

	if ($self->{Packet}->{O_R} =~ /R/) {
		$self->parse_resp($pkt);
		
	}
	else {
		if ($self->{Packet}->{OT} == "51") {
			return 0;
		}
		elsif ($self->{Packet}->{OT} == "52") {
			$self->parse_5xpkt($pkt);
			return 0;
		}
		elsif ($self->{Packet}->{OT} == "01") {
			$self->parse_0xpkt($pkt);
			return 0;
		}
		elsif ($self->{Packet}->{OT} == "54") {
			return 0;
		}
		elsif ($self->{Packet}->{OT} == "55") {
			return 0;
		}
		elsif ($self->{Packet}->{OT} == "56") {
			return 0;
		}
		else {
			return 0;
		}
	}
}


sub print_resp {
	my $self = shift;
	my $pkt;
	$pkt = join('/', ($self->{Packet}->{TRN}, $self->{Packet}->{LEN},
			$self->{Packet}->{O_R}, $self->{Packet}->{OT},
			$self->{Packet}->{ACK}, $self->{Packet}->{Magic}, 
			$self->{Packet}->{SM}));
	printf("%s\n", $pkt);
}

sub get_fileno {
	my $self = shift;
	return (fileno($self->{smsh}));
}

sub get_oper {
	my $self = shift;
	return ($self->{Packet}->{OT});
}

sub is_deliver {
	my $self = shift;
	return ((($self->{Packet}->{OT} == "01")||($self->{Packet}->{OT} == "52")) ? 1 : 0);
}

1;
